import h from 'virtual-dom/h'
import Ajaxqueue from 'ajax_queue'
import Utils from 'utils'
import AutoComplete from 'js-autocomplete'
import 'js-autocomplete/auto-complete.css'

function CityAutocomplete(args) {
	var self = this

	var o = this.options = Utils.extend({
		id: null,
		is_country: false,
		select: Utils.noop,
	}, args)

	this.value = o.value

	o.renderer.on.render.once(function() {
		self.el = document.querySelector('#' + o.id + ' .autocomplete')
		new AutoComplete({
			selector: '#' + o.id + ' .autocomplete',
			cache: false, // disabled for now because it makes the suggestion data private to the autocomplete
			source: function(request, suggest){
				self.el.classList && self.el.classList.add('ui-autocomplete-loading') // no classList in MSIE9

				Ajaxqueue({
					url: '/backend/city-lookup/',
					name: 'autocomplete', // ensure single request
					json: {
						name: request,
						members: false,
						is_country: o.is_country
					},
					logErrors: 'City lookup',
					success: function (result) {
						self.suggestions = result.results

						suggest(o.is_country
							? Object.keys(self.suggestions)
							: self.suggestions
						)
					},
					complete: clear_spinner
				})

				function clear_spinner() {
					self.el.classList && self.el.classList.remove('ui-autocomplete-loading')
				}

				clearTimeout(self.timeout)
				self.timeout = setTimeout(clear_spinner, 6000)
			},
			renderItem: function(item, search) {
				search = search.replace(/[-/\\^$*+?.()|[\]{}]/g, '\\$&amp;');
				var re = new RegExp('(' + search.split(' ').join('|') + ')', 'gi');
				return o.is_country
						? '<div class="autocomplete-suggestion" data-val="' + item + '">' + item.replace(re, '<b>$1</b>') + '</div>'
						: '<div class="autocomplete-suggestion" data-val="' + item[0] + ', ' + item[1] + '">' + (item[0] + (item[6] ? (', ' + item[6]) : '')  + ', ' + item[1]).replace(re, '<b>$1</b>')  + '</div>'
			},
			onSelect: function(e, term, item) {
				console.log('cityautocomplete.onSelect: item=', item)
				if (o.is_country) {
					o.value = item.getAttribute('data-val')
					self.value = o.value // needed for MSIE which does not fire input.onchange when selection is made
					o.select(self.suggestions[o.value])
				}else{
					var city = self.suggestions[element_index(item)]
					if(city){
						var pt = {
							lat: city[2],
							lon: city[3],
							title: city[0] + ', ' + city[1],
							long_title: city[0] + (city[6] ? (', ' + city[6]) : '')  + ', ' + city[1],
							altitude: city[5]
						}
						pt.label = pt.long_title
						pt.value = pt.title
						o.value = term
						o.select(pt)

						self.el.onchange({target: self.el}) // needed for MSIE-11
					}
				}
			}
		})
	})
}

CityAutocomplete.prototype = {
	render: function() {
		var self = this
		return h('input.autocomplete', {type: 'text', placeholder: 'Enter a city name', value: this.value, onchange: function(e){
			self.value = e.target.value // allow values that are not in the list
		}})
	}
}

function element_index(node) {
    var index = 0;
    while ((node = node.previousElementSibling)) {
        index++;
    }
    return index;
}

export default CityAutocomplete;
