define(['jquery', 'utils', 'core/clone'], function($, Utils, Clone) {

var caches = new (function CacheStore() {})(); //give the cache its own type name for profiling.

function cleanup() {
	var now = new Date();
	var empty = [];

	for (var name in caches) {
		var del = [];
		var n = 0;
		for (var key in caches[name]) {
			var k = caches[name][key];
			n++;

			if (k.timeout && k.timeout !== 'forever') {
				var age = (now.getTime() - k.inserted.getTime()) / 1000;
				if (age > k.timeout) {
					del.push(key);
				}
			}

			if (del.length == n) {
				empty.push(name);
			}

			for (var i in del) {
				//console.warn("CACHE - cleanup : " + name + " delete " + del[i] );
				delete caches[name][del[i]];
			}
		}

		for (i in empty) {
			//console.warn("CACHE - cleanup : delete " + empty[i] );
			delete caches[empty[i]];
		}
	}
}

function debug_get_object_size(obj) {
	return (typeof obj === 'undefined' || Utils.isEmptyObject(obj)) ? 0 : JSON.stringify(obj).length;
}

setInterval(cleanup, 1000 * 60 * 5);

function hash_key(key) {
	if (typeof key === 'string') {
		return key;
	}
	return Clone.makeCacheKey(key);
}

if(__DEV__){
	function handler() {
		console.log('Caches: ' + Utils.prettyJSON(caches));
	}
	document.addEventListener('debug', handler)
	window.addEventListener('unload', function() {
		document.removeEventListener('debug', handler)
	})

	function debug_miss(name, key) {
		console.warn('CACHE.GET(' + name + ',' + key + ') -> MISS')
		console.log('n_keys=', Object.keys(caches[name]).length)
		Object.keys(caches[name]).forEach(function(k, j) {
			var i = 0
			for(var i=0;i<k.length;i++){
				var a = k[i]
				var b = key[i]
				if(a !== b){
					console.log(' __ ' + i + ': ' +
						k.slice(Math.max(0, i - 10), i) + '%c' + k[i] + '%c' + k.slice(i + 1, i + 20) +
						' // ' +
						key.slice(Math.max(0, i - 10), i) + '%c' + key[i] + '%c' + key.slice(i + 1, i + 20),
						'background-color:#9f9', 'background-color:#fff',
						'background-color:#f99', 'background-color:#fff'
					)
					break
				}
			}
			console.log(' * ', j, k.length)
		})
	}
}

return {
	get: function(name, key) {

		if (!caches[name])
			caches[name] = {}

		key = hash_key(key)

		var k = caches[name][key]
		if (!k) {
			//console.warn("CACHE.GET(" + name + "," + key + ") -> MISS");
			false && __DEV__ && debug_miss(name, key)

			return
		}

		if (k.timeout && k.timeout !== 'forever') {
			var now = new Date()
			var age = (now.getTime() - k.inserted.getTime()) / 1000
			if (age > k.timeout) {
				//console.warn('CACHING(' + name + ',' + key + ') -> OBSOLETE')
				delete caches[name][key]
				return undefined
			}
		}

		//console.log("%cCACHING(" + name + "," + key + ") -> HIT", 'background-color:#9f9');
		//
		// Return a deep copy, so that user can modify original
		//
		//console.log("cacheFetch " + key + ": " + JSON.stringify(k.value));

		return Clone.cloneObject(k.value)
	},

	put: function(name, key, value, timeout) {

		if (!caches[name])
			caches[name] = {}

		key = hash_key(key)
		//console.log("cacheStore: " + key + ": " + JSON.stringify(value));
		//console.log('cacheStore: ' + (caches['ajax-queue'] ? ECMWF.n_properties(caches['ajax-queue']) : 0) + ' (' + (Math.round(debug_get_object_size(caches['ajax-queue'])/1000)) + 'k) key=' + key.substring(0, 20));

		// Store a deep copy, so that user can modify original
		//
		//console.warn("CACHING(" + name + "," + key + ") -> STORE");

		caches[name][key] = {
			value: Clone.cloneObject(value),
			timeout: timeout,
			inserted: new Date()
		};
	},

	clear: function() {
		var size = debug_get_object_size(caches);
		caches = {};
		return size;
	}
}

});
