/** @license
 *  ECMWF Clone
 *  (C) Copyright ECMWF http://www.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 */
define(['jquery', 'utils', 'time'], function($, Utils, Time) {

 function clone(obj, check_loops, depth) {
    /*
     *  check_loops - an array of the objects already tested.
     *  depth       - maximum number of allowed recursion levels.
     */
    if (obj === null) return null;
    if (obj === undefined) return undefined;
    if (depth === 0) { return '<CLONE: max depth reached.>'; }

    if (obj instanceof Date) {
        return new Date(obj);
    }

    if (obj instanceof Time) {
        return new Time(obj);
    }

    if (!Utils.isPlainObject(obj) && !Array.isArray(obj)) {
        var t = typeof obj;
        if (t === 'number' || t === 'string' || t === 'boolean') return obj
        if (Utils.isFunction(obj)) return undefined
        return undefined;
    }

    for (var i in check_loops) {
        if (obj === check_loops[i]) {
            var s = '<CLONE: loop detected: ' + obj + '>';
            console.warn(s);
            return s;
        }
    }

    if (Array.isArray(obj)) {

        var result = [];
        check_loops.push(obj);
        for (i in obj) {
            result[i] = clone(obj[i], check_loops, depth - 1);
        }
        check_loops.pop();
        return result;
    }

    if (isPlainObject(obj)) {
        result = {};
        check_loops.push(obj);
        for (i in obj) {
            var c = clone(obj[i], check_loops, depth - 1);
            if (c !== undefined) {
                result[i] = c;
            }
        }
        check_loops.pop();
        return result;
    }

    return '[un-clonable object]';
	}

	function cloneObject(object) { return clone(object, [], 50) }

	function cleanupObject(object) { return clone(object, [], 50) }

	function isPlainObject(obj) {
		if (obj.readyState) return false;
		return Utils.isPlainObject(obj);
	}

	/*
	function deepMerge(into, obj) {
		if (into === undefined) {
			return obj;
		}

		if (obj instanceof Object) {
			for (var i in obj) {
				into[i] = deepMerge(into[i], obj[i]);
			}
		} else {
			return obj;
		}

		return into;
	}
	*/

	return {
		serialise: function(object) {
			return JSON.stringify(cleanupObject(object));
		},

		makeCacheKey: function(object) {
			var result = [];

			function process(o) {
				if (Array.isArray(o)) {
					result.push('[');
					o.forEach(function(v) { process(v); });
					result.push(']');
				}
				else if (Utils.isPlainObject(o)) {
					var keys = Object.keys(o)

					keys = keys.sort();

					result.push('{')
					keys.forEach(function(k) {
						result.push(k)
						process(o[k])
					})
					result.push('}')
				} else {
					result.push(o)
				}
			}

			process(cleanupObject(object));

			return result.join(':');
		},

		cloneObject: cloneObject,
		cleanupObject: cleanupObject
	}
});
