import extend from 'extend'
import h from 'virtual-dom/h'
import {Settings} from 'settings'
import Ajaxqueue from 'ajax_queue'
import Observable from 'observable'
import Parameters from 'chart/model/parameters'
import Dashboard from 'dashboard/caching_model'
import Tabs from './tabs'
import Chart from 'eccharts-2/chart'

function DashboardSet(options) {
	var self = this

	var o = this.options = extend({
		data: null
	}, options);

	this.selection = new Observable()

	this.tabs = new Tabs({
		dashboards: o.data.dashboards,
		selection: this.selection,
		renderer: o.renderer
	})

	Token.then(function(token){
		self.selection.subscribe(function(dashboard){
			console.log('> tab select', dashboard);

			document.querySelectorAll('.ol-container').forEach(function(v){
				v.style.display = v.id === dashboard.name ? 'block' : 'none'
			})

			var ol_container = document.querySelector('#' + dashboard.name + '.ol-container')
			if(!ol_container){
				ol_container = document.createElement('div')
				document.body.appendChild(extend(ol_container, {
					id: dashboard.name,
					className: 'ol-container'
				}))
			}

			Dashboard.get(dashboard.name).then(function(result){
				extend(dashboard, result); // TODO dodgy
				dashboard.columns.forEach(function(column, c) {
					column.forEach(function(item, k) {
						Dashboard.Widget.get(item).then(function(widget) {
							widget.cache || (widget.cache = {}) // TODO check cache is not saved to backend

							widget.cache.position = {
								left: 32 + c * 300 + 'px',
								top: 140 + k * 180 + 'px'
							}

							if(widget.kind === 'chart'){
								if(!widget.cache.chart){
									var div = document.createElement('div')
									div.style.left = widget.cache.position.left
									div.style.top = widget.cache.position.top
									ol_container.appendChild(extend(div, {
										id: widget.name,
										className: 'dashboard-widget'
									}))

									var location = new Observable()
									location.set(Settings.get('last-probe-location', null))

									var time = new Parameters.time()

									var chart = new Chart({
										product: widget.data.product.name,
										target: widget.name,
										time: time,
										location: location,
										token: token
									})

									widget.cache.chart = chart
								}
							}
							o.renderer.render()
						})
					})
				})
			})
		})

		// set the selection
		// TODO this was delayed, but it turned out that delaying it was probably not needed
		var name = o.data.dashboards[o.data.selected || 0]
		Dashboard.get(name).then(function(result){
			self.selection.set(result)
		})
	})
}

DashboardSet.prototype = {
	render: function() {
		var height = 0;
		var widgets = []
		this.selection.value && this.selection.value.columns.forEach(function(column, c) {
			widgets = widgets.concat(column.map(function(v){
				if(Dashboard.widgets[v]) height = Math.max(height, parseInt(Dashboard.widgets[v].cache.position.top) + 200);
				return v
			}))
		})
		if(height) document.body.style.minHeight = height + 'px'

		return h('div', [
			this.tabs.render(),
			h('div', widgets.map(function(name){
				var widget = Dashboard.widgets[name]
				return widget
					? h('div', {style: extend({position: 'absolute'}, widget.cache.position)}, [
						widget.title,
						widget.kind === 'chart' ? undefined : h('.dashboard-widget.unhandled')
					])
					: h('div') // TODO maybe dont try and render before widgets are available?
			}))
		])
	}
}

// TODO add token to user-info
var Token = new Promise(function(resolved) {
	Ajaxqueue({
		url: '/plots/get-token/',
		dataType: 'jsonp',
		complete: function(result) {
			if(result && result.token){
				resolved(result.token)
			}else{
				console.warn('failed to get token');
			}
		}
	});
})

export default DashboardSet;
