define(['config', 'jquery', 'utils', 'virtual-dom/h', 'observable'], function(ECMWF, $, Utils, h, Observable){

var settings = {
	wheelPropagation: false
}

var Scrollbar = function(options) {
	this.options = Utils.extend({
		id: null,            // id string of 'parent' element
		height: null,        // Observable
		content_height: null // Observable
	}, options);

	var self = this, o = this.options;

	this.scroll = new Observable().set(0)
	this.scroll.subscribe(function(val) {
		var scrollable = document.querySelector('#' + o.id + ' .scrollable')
		if(scrollable){
			scrollable.scrollTop = val
		}
	})

	this.state = {
		inner_height: 0 // handle size
	}

	var getEventClassName = (function() {
		var incrementingId = 0;
		return function() {
			var id = incrementingId;
			incrementingId += 1;
			return '.perfect-scrollbar-' + id;
		};
	}())

	this.eventClassName = getEventClassName();

	o.content_height.subscribe(function(){
		self.state.inner_height = Math.max(16, o.height.value * o.height.value / o.content_height.value);

		//var max_scrolltop = o.content_height.value - o.height.value;
	})
}

Scrollbar.prototype = {
	render: function() {
		if(this.state.inner_height >= this.options.height.value) return this.bind_done = false, []

		this.bind_done || this.bind_mouse()
		this.bind2_done || this.bind_mousewheel()
		this.bind3_done || this.bind_touch()

		//console.log('scrollbar.render: scrolltop=', this.scroll.value, ' height=', this.options.height.value + '/' + this.options.content_height.value + ' top=', this._inner_top(), ' inner=', this.state.inner_height);

		return h(
			'.ps-scrollbar-y-rail',
			{style: {
				top: 0,
				bottom: 0,
				backgroundColor: '#eee',
				opacity: 0.6
			}},
			[
				h('.ps-scrollbar-y', {style: {top: this._inner_top() + 'px', height: this.state.inner_height + 'px'}})
			]
		)
	},

	update_bar_size_and_position: function() {
		var scrollable = document.getElementById(this.options.id).querySelector('.scrollable');
		Utils.css(scrollable.parentElement.querySelector('.ps-scrollbar-y'), {top: this._inner_top(), height: this.state.inner_height})
	},

	_inner_top: function() {
		var o = this.options;
		if(!o.content_height.value) return 0;

		var pct = this.scroll.value / (o.content_height.value - o.height.value);
		return (o.height.value - this.state.inner_height) * pct;
	},

	bind_mouse: function() {
		var self = this, o = this.options, y0, current_page_y;
		var scrollable = document.querySelector('#' + o.id + ' .scrollable')

		if(scrollable && ((this.rail = scrollable.parentElement.querySelector('.ps-scrollbar-y-rail')))){
			this.handle = this.rail.querySelector('.ps-scrollbar-y')

			this.handle.addEventListener('mousedown', function(e) {
				y0 = Utils.position(self.handle).top
				current_page_y = e.pageY
				// TODO why does self.rail become null during testing?
				self.rail && self.rail.classList.add('in-scrolling');
				e.stopPropagation();
				e.preventDefault();
			})

			this.rail.addEventListener('mouseup', function(e) {
				if(e.target === self.rail){
					var dy = self.state.inner_height * (e.pageY - Utils.offset(scrollable).top < self._inner_top() ? -1 : +1)
					dy && scrollto(self._inner_top() + dy)
					return false
				}
			})

			// TODO bind in mousedown handler instead
			document.addEventListener('mousemove', function(e) {
				if (self.rail && self.rail.classList.contains('in-scrolling')) {
					var dy = e.pageY - current_page_y
					scrollto(y0 + dy)
					return false
				}
			})

			document.addEventListener('mouseup', function(e) {
				//if (self.rail.classList.contains('in-scrolling')) {
					self.rail.classList.remove('in-scrolling');
				//}
			})

			this.bind_done = true

			function scrollto(pos) {
				var y = Math.max(0, Math.min(o.height.value - self.state.inner_height, pos))
				var val = Math.round(y * (o.content_height.value - o.height.value) / (o.height.value - self.state.inner_height))

				scrollable.scrollTop = val;
				val = scrollable.scrollTop

				//self.update_bar_size_and_position();
				Utils.css(self.handle, {top: y})

				self.scroll.set(val)
			}
		}
	},

	scroll_to: function(pos) {
		// Note that if the scrollable is not visible, then at some pointer after it is
		// made visible, the browser will restore the previous scrollTop and emit a scroll
		// event.
		// It is very important that this.scroll.value is in sync with scrollTop otherwise the
		// contents will not be visible.
		var self = this, o = this.options

		this.scroll.set(Math.min(pos, o.content_height.value - o.height.value))

		o.renderer.on.render.subscribe(self.options, function() {
			var t;
			var scrollable = self.scrollable = document.querySelector('#' + o.id + ' .scrollable');
			__DEV__ && !Utils.is_visible(scrollable) && console.warn('scrollable is not visible');

			function on_visible() {
				if(t) t = clearTimeout(t)
				scrollable.scrollTop = self.scroll.value
				//self.scroll.set(self.scroll.value)
				self.scroll.emit(self.scroll.value) // handlers must run even if value has not changed
			}

			if(Utils.is_visible(scrollable)){
				on_visible()
			}else{
				Utils.once(scrollable, 'scroll', on_visible)

				// there will be no scroll event if the browser has no scrollTop to restore
				if(self.scroll.value)
					t = setTimeout(on_visible, 1000)
			}

			setTimeout(function() {
				o.renderer.on.render.unsubscribe(self.options);
			}, 0)
		})
	},

	bind_mousewheel: function() {
		var self = this;
		var shouldPrevent = false;
		var scrollable = document.querySelector('#' + this.options.id + ' .scrollable');
		if(!scrollable) return;
		this.bind2_done = true;

		var getDeltaFromEvent = function (e) {
			var deltaY = -1 * e.originalEvent.deltaY;

			if (typeof deltaY === 'undefined') {
				// OS X Safari
				deltaY = e.originalEvent.wheelDeltaY / 6;
			}

			return deltaY;
		};

		var mousewheelHandler = function (e) {
			var delta = getDeltaFromEvent(e)
			var old_top = scrollable.scrollTop;

			shouldPrevent = false;
			scrollable.scrollTop = old_top - (delta * ECMWF.wheelspeed);
			var val = scrollable.scrollTop
			if(val !== old_top){
				//shouldPrevent = true;

				self.update_bar_size_and_position(); // update bar position imediately

				self.scroll.set(val)
			}

			shouldPrevent = (shouldPrevent || self.shouldPreventDefault(delta));
			if (shouldPrevent) {
				e.stopPropagation();
				e.preventDefault();
			}
		}

		if (typeof window.onwheel !== 'undefined') {
			$(scrollable).on('wheel' + self.eventClassName, mousewheelHandler);
		} else if (typeof window.onmousewheel !== 'undefined') {
			$(scrollable).on('mousewheel' + self.eventClassName, mousewheelHandler);
		}

		// fix Firefox scroll problem
		$(scrollable).on('MozMousePixelScroll' + self.eventClassName, function (e) {
			if (shouldPrevent) {
				e.preventDefault();
			}
		});
	},

	bind_touch: function() {
		var self = this;
		var scrollable = $('#' + this.options.id).find('.scrollable');
		if(!scrollable.length) return;
		this.bind3_done = true;

		var applyTouchMove = function (differenceX, differenceY) {
			var val = scrollable.scrollTop() - differenceY
			scrollable.scrollTop(val);

			self.update_bar_size_and_position(); // update bar position imediately
			self.scroll.set(val)
		}

		var startCoords = {}, startTime = 0, speed = {}, breakingProcess = null, inGlobalTouch = false;

		$(window).on('touchstart' + self.eventClassName, function (e) {
			console.log('global touchstart');
			inGlobalTouch = true;
		})

		$(window).on('touchend' + self.eventClassName, function (e) {
			console.log('global touchend');
			inGlobalTouch = false;
		})

		scrollable.on('touchstart' + self.eventClassName, function (e) {
			var touch = e.originalEvent.targetTouches[0];

			startCoords.pageX = touch.pageX;
			startCoords.pageY = touch.pageY;

			startTime = (new Date()).getTime();

			if (breakingProcess !== null) {
				clearInterval(breakingProcess);
			}

			e.stopPropagation();
		})

		scrollable.on('touchmove' + self.eventClassName, function (e) {
			if (!inGlobalTouch && e.originalEvent.targetTouches.length === 1) {
				var touch = e.originalEvent.targetTouches[0];

				var currentCoords = {};
				currentCoords.pageX = touch.pageX;
				currentCoords.pageY = touch.pageY;

				var differenceX = currentCoords.pageX - startCoords.pageX,
				differenceY = currentCoords.pageY - startCoords.pageY;

				applyTouchMove(differenceX, differenceY);
				startCoords = currentCoords;

				var currentTime = (new Date()).getTime();

				var timeGap = currentTime - startTime;
				if (timeGap > 0) {
					speed.x = differenceX / timeGap;
					speed.y = differenceY / timeGap;
					startTime = currentTime;
				}

				e.preventDefault();
			}
		});

		scrollable.on('touchend' + self.eventClassName, function (e) {
			clearInterval(breakingProcess);
			breakingProcess = setInterval(function () {
				if (Math.abs(speed.x) < 0.01 && Math.abs(speed.y) < 0.01) {
					clearInterval(breakingProcess);
					return;
				}

				applyTouchMove(speed.x * 30, speed.y * 30);

				speed.x *= 0.8;
				speed.y *= 0.8;
			}, 10);
		});
	},

	shouldPreventDefault: function (deltaY) {
		var o = this.options;

		var scrollTop = $('#' + this.options.id).scrollTop(); // TODO looks like this is checking the wrong element
		if ((scrollTop === 0 && deltaY > 0) || (scrollTop >= o.content_height.value - o.height.value && deltaY < 0)) {
			return !settings.wheelPropagation;
		}

		if (deltaY === 0) {
			return false;
		}
		return true;
	}
}

return Scrollbar

})
