/*
 *  TODO
 *    - most of this code is shared with other views
 *    - improve left and right scrolling
 */
import $ from 'jquery'
import Ajaxqueue from 'ajax_queue'
import Utils from 'utils'
import h from 'virtual-dom/h'
import page from 'chart-app/router'
import ViewBase from './base'
import 'perfect-scrollbar/perfect-scrollbar.css'
import './horizontal.css'

var show_title = true;
//var img_data = {}

var HorizontalView = function(options) {
	this.create(options)
}

HorizontalView.prototype = Utils.extend({}, ViewBase, {

	create: function(options) {
		this.options = Utils.extend({
			renderer: undefined,
			products: null,
			width: 250, // a default value is needed but the details view is full width so the user does not need to specify the width
			height: 400,
			scrollbar: null
		}, options);

		ViewBase.create.apply(this, arguments);

		var self = this, o = this.options;

		this.item_height = 64 + (show_title ? 30 : 0);
		this.item_width = 127;
		this.padding = {x: 8, y: 16};
		this.overscan = 1             // assign extra rows to allow for scrolling.
		this.group_label_height = 0;
		this.megatooltips = false
		this.eventNamespace = 'horizontal'

		o.scrollbar.state.inner_height = 20 // TODO

		o.renderer.on.render.subscribe(self.options, function(){
			var el = o.renderer.root // TODO use id
			var scrollable = el.querySelector('.scrollable')
			var visible = Utils.is_visible(el.querySelector('.catalogue-grid')) // scrollTop is not available if the element is hidden
			if(scrollable.length && visible){
				setTimeout(function() {
					o.renderer.on.render.unsubscribe(self.options);
				}, 0)

				o.renderer.render()
			}
		})

		o.products.subscribe(function(){
			self.cache.natural_height = null;
		})
	},

	/*
	 *  Scroll to an item
	 */
	scroll_to: function(a) {
		if(__DEV__ && typeof a === 'undefined') return console.error('scroll_to: missing index');
		var o = this.options
		var item = o.products.items[a]
		var scrolltop = o.scrollbar.scroll.value;
		if(item[this.eventNamespace]){
			var y = item[this.eventNamespace].y
			if(!y || y < scrolltop || y > scrolltop + this.options.height){
				// note: we cannot set el.scrollTop because it may not yet be visible
				o.scrollbar.scroll_to(y)
			}
		}
	},

	_assign_fresh: function() {
		// dont know if these are relevant with vdom
		this._on_n_viewable_items_changed();
		this.have.assignment.resolve();
	},

	_invalidate_positions: function() {
		this._positions_valid = false;
	},

	_calc_positions: function() {
		var self = this, o = this.options;
		//if(!self.n_columns) return console.log('calc_positions: size not set')

		this.groups_valid || this._update_groups();

		var label_height = (self.groups.length > 1) ? self.group_label_height : 0;

		var scrolltop = o.scrollbar.scroll.value
		/*
		if(scrolltop > self.natural_height()){
			console.warn('content_height is wrong');
			self.content_height.set(scrolltop);
		}
		*/

		var found = false;
		var y = -(self.item_height + self.padding.y);
		self._results_by_group_all(
			function(group){
				y += self.item_height + self.padding.y + 8;
				group.y = y;
			},
			function(item, i, ig, gr){
				y = gr.y + label_height + Math.floor((ig / self.n_columns)) * (self.item_height + self.padding.y);
				gr.height = label_height + Math.ceil(((ig + 1) / self.n_columns)) * (self.item_height + self.padding.y) + 8;

				var overscan = self.overscan * self.item_height;
				var visible =
					(y > scrolltop - overscan) &&
					(y < scrolltop + o.height + overscan);

				if(!found && visible){
					found = true;
					self._first = i;
				}

				item[self.eventNamespace] = item[self.eventNamespace] || {};
				Utils.extend(item[self.eventNamespace], {
					ig: ig,
					y: y,
					visible: visible
				})
			}
		)
		this._positions_valid = true;
	},

	render: function() {
		var self = this, o = self.options
		//console.log('%chorizontal.render', 'background-color:#ff9')

		o.products.cancel_thumbnails();

		this.groups_valid || this._update_groups();
		this._positions_valid || self._calc_positions();

		var labels = [];
		var items = [];

		self._results_by_group(
			function(group){
				if(self.groups.length > 1){
					labels.push(h('h3', {
						style: {top: (group.y - 8) + 'px'},
						onclick: function(e){
							page.add_state();
							var facets = o.products.lookup_filter_by_name('facets')
							facets && facets.set(['Product group', group.name], true)
							e.stopPropagation()
						}
					},
					[group.name]));
				}
			},
			function(item, m, d, ig, gr){
				var local = item[self.eventNamespace];

				var column = ig % self.n_columns;

				//var url = img_data[item.name()] || item.cache.thumbnail.call(item); // cannot use the normal thumbnail method here because it is asynchronous

				// TODO change selected2 -> selected once the old grid is removed
				items.push(h('.item.thumbnail-wrapper' + (m === o.products.selection.value ? '.selected2' : ''), {style: {top: local.y + 'px', left: column * (self.item_width + self.padding.x) + 'px'}}, [
					h('div.thumbnail', [
						h('img.thumbnail-img' + (item.thumbnail_error ? '.failed.ui-icon.ui-icon-alert' : ''), {
							alt  : item.title_(),
							title: self.megatooltips ? null : $('<div>').html(item.data.description).text() || item.title_(),
							name : item.name(),
							src  : Ajaxqueue.options.host + '/backend/get-thumbnail/' + o.products.kind + '/' + item.name() + '/',
							ondragstart: function(e){ e.preventDefault() }
							/*
							style: url || item.thumbnail_error
								? { visibility: 'visible', opacity: 1.0 }
								: { visibility: 'hidden', opacity: 0.0 }
							*/
						})
					]),
					h('div.col-title', item.title_())
				]));

				/*
				!url && !item.thumbnail_error && item.thumbnail && item.thumbnail({tag: item.name() + self.eventNamespace, priority: 'very-high'}, function(url, image, error){
					if(error){
						item.thumbnail_error = true;
					}else{
						if(!image) return;

						// we may have multiple callbacks from the same request, so img_data may already be set
						img_data[item.name()] = image;
					}
					self.options.renderer.render();
				});
				*/
			}
		);

		items.push(h('div.group-labels', labels));

		return [
			h('.grid-items.horizontal.thumbs.unselectable', {
				style: {height: (o.height ? this.natural_height() : 400) + 'px'},
				onclick: function(ev){
					console.log('horizontal.click');
					var target = $(ev.target).closest('.item')
					var selected = target.hasClass('selected2');
					target = target.find('.thumbnail-img');

					var item = o.products.product_by_name(target.attr('name'));
					item && !selected && o.select(item)
					item && o.click(item);
				}
			}, items),
			this.listnav('left'),
			this.listnav('right')
		]
	},

	listnav: function(t) {
		var self = this, o = this.options

		var disabled = (t === 'left') && (o.scrollbar.scroll.value < 1)

		return h('.list-nav.' + t + (disabled ? '.disabled' : ''), {onclick: function(e){
				o.scrollbar.scroll.set(Math.max(0, o.scrollbar.scroll.value + (self.item_height + self.padding.y) * (t === 'left' ? -1 : 1)))
			}},
			h('div')
		)
	},

	select: function(item) {
		var self = this, list = this.options.products;

		list.have.filtering.done(function() {
			if(!item[self.eventNamespace]) item[self.eventNamespace] = {}; // do this earlier?
			self.options.renderer.render();
		});
	},

	scroll: function(top) {
		var self = this, o = this.options;

		self._invalidate_positions();
		o.renderer.render();
	},

	_update_groups: function() {
		var self = this, list = this.options.products, group;

		self.groups = [];
		for(var i=0;i<list.filtered.length;i++){
			var item = list.items[list.filtered[i]];
			var g = ((item.data.tags || [])['Product group'] || [])[0] || 'Other';
			if(g !== group){
				self.groups.push({name: group = g, i: i});
			}
		}
		//console.log('* _update_groups: groups=', self.groups.length, self.groups);

		this.groups_valid = true;
		self._positions_valid = false;
		return self.groups.length > 1;
	},

	_each_allocated: function(fn) {
		var self = this, o = self.options, list = o.products;

		self._positions_valid || self._calc_positions();

		for(var i=self._first;i<list.filtered.length;i++){
			var m = list.filtered[i];
			var item = list.items[list.filtered[i]];
			var local = item[self.eventNamespace];
			//if(!local.visible) console.log('each_allocated: range ', self._first, i, ' (' + (i - self._first) + ')');
																// why needed? happens when clicking on dashboard link after adding widget
																if(!local) break;
			if(!local.visible) break;

			fn(item, m, i);
		}
	},

	// only results in viewport
	_results_by_group: function(fn1, fn2) {
		var self = this;

		self._positions_valid || self._calc_positions();

// TODO g may be incorrect if scrolling
		var g = -1, gr = {};
		this._each_allocated(function(item, m, d){
			var local = item[self.eventNamespace];
			var ig = local.ig;
			var group_name = ((item.data.tags && item.data.tags['Product group'] || [])[0]) || 'Other';
			if(g < 0 || (group_name && group_name !== gr.name)){
				g++;
				gr = Utils.findWhere(self.groups, {name: group_name});
				fn1(gr);
			}
			fn2(item, m, d, ig++, gr, g);
		});
	},

	_results_by_group_all: function(fn1, fn2, done) {
		var self = this, list = this.options.products;

		var g = -1, ig = 0, gr = {};
		for(var i=0;i<list.filtered.length;i++){
			var item = list.items[list.filtered[i]];
			var group_name = ((item.data.tags && item.data.tags['Product group'] || [])[0]) || 'Other';
			if(!i || (group_name !== gr.name)){
				g++;
				gr = Utils.findWhere(self.groups, {name: group_name});
				fn1(gr);
				ig = 0;
			}
			fn2(item, i, ig++, gr, g);
		}
		done && done(gr);
	},

	// return maximum number of items that can be shown when scrolled to top.
	max_items: function() {
		var self = this, o = this.options;
		var max = 0;

		if(this.cache.max_items) return this.cache.max_items;

		var y = 0;
		o.products.each_by_facet('Product group',
			function(group){
				y += self.group_label_height;
			},
			function(item, i, ig){
				if(ig % self.n_columns === 0){
					y += self.item_height + self.padding.y;
				}
				if(y < self.cache.max_height){
					max = i + 1;
				}else{
					// TODO list.each_by_facet should respect return value of false
					return false;
				}
			}
		)

		return this.cache.max_items = max;
	},

	// return the height of all the filtered items
	natural_height: function() {

		if(this.cache.natural_height) return this.cache.natural_height;

		this.groups_valid || this._update_groups();
		this._positions_valid || this._calc_positions();

		var h = 0;
		this.groups.forEach(function(gr) {
			if(gr.height) h += gr.height;
		})

		return this.cache.natural_height = h;
	},

	_new_element: function(d) {
		return {};
	}
})

export default HorizontalView;

