/*!
 * Bootstrap without jQuery v0.6.1 for Bootstrap 3
 * By Daniel Davis under MIT License
 * https://github.com/tagawa/bootstrap-without-jquery
 */

/*
 *  TODO currently only used for drop down menus. The code for alerts and dialogues may not be needed
 *  TODO implement resetHeaderNav()
 *  TODO implement resetWithEscKey()
 */

var transitionend;

// Get the potential max height of an element
function getMaxHeight(element) {
	// Source: http://n12v.com/css-transition-to-from-auto/
	var prevHeight = element.style.height;
	element.style.height = 'auto';
	var maxHeight = getComputedStyle(element).height;
	element.style.height = prevHeight;
	element.offsetHeight; // force repaint
	return maxHeight;
}

/*
 * Collapse action
 * 1. Get list of all elements that are collapse triggers
 * 2. Add click event listener to these elements
 * 3. When clicked, change target element's class name from "collapse" to "collapsing"
 * 4. When action (collapse) is complete, change target element's class name from "collapsing" to "collapse in"
 * 5. Do the reverse, i.e. "collapse in" -> "collapsing" -> "collapse"
 */

// Start the collapse action on the chosen element
function doCollapse(event) {
	event.preventDefault();
	var targets = getTargets(event);
	var dataTarget = targets.dataTarget;

	// Add the "in" class name when elements are unhidden
	if (dataTarget.classList.contains('in')) {
		hide(dataTarget, targets.evTarget);
	} else {
		show(dataTarget, targets.evTarget);
	}
	return false;
}

// Show a target element
function show(element, trigger) {
	element.classList.remove('collapse');
	element.classList.add('collapsing');
	trigger.classList.remove('collapsed');
	trigger.setAttribute('aria-expanded', true);
																	// TODO move to css
																	element.style.display = 'block';

	// Set element's height to its maximum height
	element.style.height = getMaxHeight(element);

	// Call the complete() function after the transition has finished
// TODO why the transition isnt working?
	if (false && transitionend) {
		element.addEventListener(transitionend, function() {
			complete(element);
		}, false);
	} else {
		// For browsers that don't support transitions (e.g. IE9 and lower);
		complete(element);
	}
}

// Hide a target element
function hide(element, trigger) {
	element.classList.remove('collapse');
	element.classList.remove('in');
	element.classList.add('collapsing');
	trigger.classList.add('collapsed');
	trigger.setAttribute('aria-expanded', false);
																	element.style.display = 'none';

	// Reset element's height
	element.style.height = getComputedStyle(element).height;
	element.offsetHeight; // force repaint
	element.style.height = '0px';
}

// Change classes once transition is complete
function complete(element) {
	element.classList.remove('collapsing');
	element.classList.add('collapse');
	element.setAttribute('aria-expanded', false);

	// Check whether the element is unhidden
	if (element.style.height !== '0px') {
		element.classList.add('in');
		element.style.height = 'auto';
	}
}

// Get an event's target element and the element specified by the "data-target" attribute
function getTargets(event) {
	var targets = {};
	event = event || window.event;
	targets.evTarget = event.target//event.currentTarget || event.srcElement;
	var dataTarget = targets.evTarget.getAttribute('data-target');
	targets.dataTarget = (dataTarget) ? document.querySelector(dataTarget) : false;
	return targets;
}

window.addEventListener('load', function(event) {

	// transitionend - source: https://stackoverflow.com/questions/5023514/how-do-i-normalize-css3-transition-functions-across-browsers#answer-9090128
	function transitionEndEventName() {
		var i
		var el = document.createElement('div')
		var transitions = {
				transition:'transitionend',
				MozTransition:'transitionend',
				WebkitTransition:'webkitTransitionEnd'
		}

		for (i in transitions) {
			if (transitions.hasOwnProperty(i) && el.style[i] !== undefined) {
			return transitions[i];
			}
		}

		return false;
	}
	transitionend = transitionEndEventName();

	// Fire a specified event
	// Source: http://youmightnotneedjquery.com/
	function fireTrigger(element, eventType) {
		if (document.createEvent) {
			var event = document.createEvent('HTMLEvents')
			event.initEvent(eventType, true, false)
			element.dispatchEvent(event)
		} else {
			element.fireEvent('on' + eventType)
		}
	}

	/*
	 * Alert dismiss action
	 * 1. Get list of all elements that are alert dismiss buttons
	 * 2. Add click event listener to these elements
	 * 3. When clicked, find the target or parent element with class name "alert"
	 * 4. Remove that element from the DOM
	 */

	// Start the collapse action on the chosen element
	function doDismiss(event) {
		event.preventDefault();
		// Get target element from data-target attribute
		var targets = getTargets(event);
		var target = targets.dataTarget;

		if (!target) {
			// If data-target not specified, get parent or grandparent node with class="alert"
			var parent = targets.evTarget.parentNode;
			if (parent.classList.contains('alert')) {
				target = parent;
			} else if (parent.parentNode.classList.contains('alert')) {
				target = parent.parentNode;
			}
		}

		fireTrigger(target, 'close.bs.alert');
		target.classList.remove('in');

		function removeElement() {
			// Remove alert from DOM
			try {
				target.parentNode.removeChild(target);
				fireTrigger(target, 'closed.bs.alert');
			} catch(e) {
				window.console.error('Unable to remove alert');
			}
		}

		// Call the complete() function after the transition has finished
		if (transitionend && target.classList.contains('fade')) {
			target.addEventListener(transitionend, function() {
				removeElement();
			}, false);
		} else {
			// For browsers that don't support transitions (e.g. IE9 and lower);
			removeElement();
		}

		return false;
	}

    // Get all alert dismiss buttons and add click event listeners
    var dismissList = document.querySelectorAll('[data-dismiss=alert]');
    for (var j = 0, lenj = dismissList.length; j < lenj; j++) {
        dismissList[j].onclick = doDismiss;
    }

	/*
	 * Dropdown action
	 * 1. Get list of all elements that are dropdown triggers
	 * 2. Add click and blur event listeners to these elements
	 * 3. When clicked, add "open" to the target element's class names, or remove if it exists
	 * 4. On blur, remove "open" from the target element's class names
	 */

	// Close a dropdown menu
	/*
	function closeDropdown(event) {
		event = event || window.event;
		var evTarget = event.currentTarget || event.srcElement;
		evTarget.parentElement.classList.remove('open');

		// Trigger the click event on the target if it not opening another menu
		if(event.relatedTarget && event.relatedTarget.getAttribute('data-toggle') !== 'dropdown') {
			event.relatedTarget.click();
		}
		return false;
	}
	*/

	// TODO may have to add 'tabindex', '0' to markup for chrome

	// Set event listeners for dropdown menus
	document.addEventListener('click', function(e) {
		var element = e.target.hasAttribute
			? e.target.hasAttribute('data-toggle')
				? e.target
				: e.target.parentElement
			: null

		if(element){
			var attr = element.getAttribute('data-toggle')

			switch(attr){
				// show dropdown menus
				case 'dropdown':
					//e = e || window.event
					element.parentElement.classList.toggle('open')

					break
				case 'collapse':
					doCollapse(e)
			}

			[].slice.call(document.querySelectorAll('.dropdown-toggle')).forEach(function(el){
				if(el !== element){
					el.parentElement.classList.remove('open')
				}
			})

			if(e.target.classList.contains('navbar-toggle')){
				var el = e.target
				el.classList.toggle('active')

				// overlay
				overlay.style.display =
					el.classList.contains('active')
						? 'block'
						: 'none'
			}

			return false
		}
	})

	var overlay = document.querySelector('.menu-collapse-overlay')
	overlay.addEventListener('click', function(e) {
		var button = document.getElementById('toggle-menu')
		hide(document.querySelector('.header-menu.collapse.in'), button);
		button.classList.remove('active');
		overlay.style.display = 'none'
	})
})

export {show, hide, doCollapse}
