
define(['config', 'extend', './utils.css'], function(ECMWF, extend) {

var class2type = {}
var toString = class2type.toString
var hasOwn = class2type.hasOwnProperty
var fnToString = hasOwn.toString
var ObjectFunctionString = fnToString.call(Object)

var space = function(n) {
	var s = "";
	while(n > 0) { s += " "; n--; }
	return s;
}

var pretty = function(obj, depth)
{
	if(depth/3 > 5)  { return "..."; }

	if(typeof(obj) == 'object'){
		var list = []
		var s = space(depth+3);
		if (obj instanceof Array) {
			var bra = '['; var ket = ']';
			for (var n in obj) {
				var value = pretty(obj[n],depth+3);
				list.push(s + value);
			}
		} else {
			bra = '{'; ket = '}';
			for (n in obj) {
				value = pretty(obj[n],depth+3);
				list.push( s + '"' + n + '"=' + value);
			}
		}

		return bra + (list.length > 0?"\n":"") + list.join(",\n") + (list.length > 0?"\n" + space(depth) :"") + ket;
	}

	if (typeof obj == 'string')
		return '"' + obj + '"';

	return "" + obj;
}


/*
ECMWF.json_to_html = function(obj){

	function do_child(child){
		depth++;
		depth_used = Math.max(depth_used, depth);
		var n = 0;
		for(var propName in child){
			if(depth < max_recursion){
				if(typeof child[propName] === 'object'){
					if($.isPlainObject(child[propName])){
						add_line(propName);
						do_child(child[propName]);
					}
					else add_line(propName + ' ***');
				}
				else add_line(propName);
			}else{
				add_line(propName);
				add_line('[max recursion!! ' + depth + ']');
			}
			n++;
		}
		if(!n) add_line('[no properties]');
		depth--;

		function add_line(s){
			output.push('<div style="margin-left:' + (depth * 10) + 'px">' + s + '</div>');
		}
	}

	var depth = 0;
	var max_recursion = 10;
	var depth_used = 0;
	var output = [];

	do_child(obj);

	return output.join('') + 'depth=' + depth_used;
}

ECMWF.print_element = function(el)
{
	//useful when firebug not available.

	if(typeof(el) == 'undefined') return 'undefined';
	if(el === null) return 'null';

	var o = '';
	if(el.length && ECMWF.isset(el.attr)){
		o += ' $tag=' + el[0].tagName + ' id=\'' + el.attr('id') + '\' class=\'' + el.attr('class') + '\' parent_id=\'' + el.parent().attr('id') + '\' parent_class=\'' + el.parent().attr('class') + '\'';
	}else if(typeof el == 'object' && ECMWF.isset(el.scrollWidth)){
		o += ' html-element: ' + el.tagName + ' class="' + el.className + '" id=' + el.id + '. nodeType=' + el.nodeType + '.';
	}else if(ECMWF.isset(el.nodeName) && el.nodeName == '#document'){
		o += ' #document.';
	}else if(el.parent && el.parent.constructor.toString() == '[object Window]'){
		o += ' window';
	}else if(typeof el == 'object'){
		o += ' object: contructor=' + el.constructor + '. '+ Object.keys(el).join(', ');
	}else if(typeof el == 'function'){
		o += ' type=function'
	}else if(typeof el == 'number'){
		o += ' type=number (' + el + ')'
	}else{
		o += ' dom element? len=' + el.length + ' el=' + el;
	}
	return o;
}

ECMWF.fn_name = function(fn)
{
	var name = /\W*function\s+([\w\$]+)\(/.exec(fn);
	return name ? name[1] : 'anonymous';
}
*/

ECMWF.pad = function(str, len, pad)
{
	if(typeof str === 'undefined') str = '';
	if(typeof str != 'string'){ str = str.toString(); }
	if(typeof len == 'undefined'){ len = 0; }
	if(typeof pad == 'undefined'){ pad = ' '; }

	if(len + 1 >= str.length){
		str = Array(len + 1 - str.length).join(pad) + str;
	}
	return str;
}

ECMWF.truncate = function(str, len){
	if(str && str.length > len){
		var trunc = str.substring(0, len);
		return trunc.replace(/\w+$/, '') + '&hellip;';
	}
	else return str;
}

ECMWF.isset = function(a){
	if(typeof a == 'undefined') return false;
	if(typeof a == 'string' && !a) return false;
	if(typeof a == 'number' && isNaN(a)) return false;
	if(a === null) return false;
	return (a != 'undefined');
}

ECMWF.is_empty = function(obj){
	var i;
	for(i in obj) return false;
	return true;
}

ECMWF.format_date = function(time) {
	if(!ECMWF.isset(time)) return '';
	time = '' + time;

	var d = time.substring(0, 8);
	var t = time.substring(8, 12);
	return d + ' ' + t + ' UTC';
}

/* increases the size of an object by mb megabytes to debug memory problems */
ECMWF.inflate = function (obj, mb) {
	mb = mb || 1;
	obj.padding = new Array(1000000 * mb + 1).join('_');
}

ECMWF.add_links = function (input) {
	return input
		.replace(/(ftp|http|https|file):\/\/[\S]+(\b|$)/gim, '<a href="$&" class="my_link" target="_blank">$&</a>')
		.replace(/([^/])(www[\S]+(\b|$))/gim, '$1<a href="http://$2" class="my_link" target="_blank">$2</a>');
}

ECMWF.remove = function (obj, name) {
	var val = obj[name];
	delete obj[name];
	return val;
}


ECMWF.timer = function(milliseconds)
{
	this.milliseconds = milliseconds;
	this.id = null;

	this.start = function(done){
		var self = this;
		__DEV__ && this.id && console.warn('timer.start: already started');
		this.id = setTimeout(function(){
			self.id = null;
			done();
		}, this.milliseconds);
	}

	this.clear = function(done){
		clearTimeout(this.id);
		this.id = null;
	}
}


ECMWF.xhr_to_error_string = function(xhr)
{
	if(!xhr.error) return 'Unknown error';

	return xhr.error.length
		? xhr.error[0]
		: xhr.error;
}


var Utils = {
	blank_gif: 'data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==',

	/*
	 *  Return true if page configuration indicates use of a different host for ajax requests.
	 */
	same_origin: function() {
		var loc = window.location.protocol + '//' + window.location.hostname + (window.location.port ? (':' + window.location.port) : '');
		var backend_host
		if(ECMWF.options.backend_host){
			backend_host = Array.isArray(ECMWF.options.backend_host)
				? ECMWF.options.backend_host[0]
				: ECMWF.options.backend_host
		}
		var same_origin = (!backend_host) || (loc === backend_host)
		same_origin || console.log('same_origin=', same_origin,' backend=', loc + ' ', backend_host)

		return same_origin;
	},

	n_properties: function(obj) {
		var i = 0, propName; // eslint-disable-line no-unused-vars
		for(propName in obj) i++;
		return i;
	},

	ucfirst: function(str) {
		str += '';
		return str.charAt(0).toUpperCase() + str.slice(1);
	},

	prettyJSON: function(obj, indent) {
		return pretty(obj, 0);
	},

	escape_regex: function(s) {
		return s.replace(/[.*+?^${}()|[\]/\\]/g, '\\$0');
	},

	camel_to_dashes: function(str){
		return str.replace( /([a-z])([A-Z])/g, '$1-$2' ).toLowerCase();
	},

	/*
	 *  Execute a list of non-async functions in series and return a list of return values
	 */
	run: function(list, args) {
		var self = this;
		return list.map(function(fn) {
			return fn.apply(self, args);
		})
	},

	/*
	 *   From jquery
	 */
	isFunction: function(obj) {

		// Support: Chrome <=57, Firefox <=52
		// In some browsers, typeof returns "function" for HTML <object> elements
		// (i.e., `typeof document.createElement( "object" ) === "function"`).
		// We don't want to classify *any* DOM node as a function.
		return typeof obj === 'function' && typeof obj.nodeType !== 'number';
	},

	/*
	 *   From jquery
	 */
	isPlainObject: function(obj) {
		var proto, Ctor

		// Detect obvious negatives
		// Use toString instead of jQuery.type to catch host objects
		if (!obj || toString.call(obj) !== '[object Object]') {
			return false
		}

		proto = Object.getPrototypeOf(obj)

		// Objects with no prototype (e.g., `Object.create( null )`) are plain
		if (!proto) {
			return true
		}

		// Objects with prototype are plain iff they were constructed by a global Object function
		Ctor = hasOwn.call(proto, 'constructor') && proto.constructor;
		return typeof Ctor === 'function' && fnToString.call(Ctor) === ObjectFunctionString
	},

	/*
	 *   From jquery
	 */
	isEmptyObject: function(obj) {
		var name;

		for(name in obj){
			return false;
		}
		return true;
	},

	/*
	 *   From jquery
	 */
	extend: function() {
		var options, name, src, copy, copyIsArray, clone,
			target = arguments[ 0 ] || {},
			i = 1,
			length = arguments.length,
			deep = false;

		if (typeof target === 'boolean') {
			deep = target;

			target = arguments[i] || {};
			i++;
		}

		if (typeof target !== 'object' && !Utils.isFunction(target)) {
			target = {};
		}

		for(;i < length;i++) {

			if((options = arguments[i]) !== null){

				for(name in options){
					src = target[name];
					copy = options[name];

					if(target === copy){
						continue;
					}

					if(deep && copy && (Utils.isPlainObject(copy) || (copyIsArray = Array.isArray(copy)))){

						if(copyIsArray){
							copyIsArray = false;
							clone = src && Array.isArray(src) ? src : [];

						} else {
							clone = src && Utils.isPlainObject(src) ? src : {};
						}

						target[name] = extend(deep, clone, copy);

					} else if(copy !== undefined){
						target[ name ] = copy;
					}
				}
			}
		}

		return target;
	},

	/*
	 *   From jquery
	 */
	is_visible: function(elem) {
		if(!elem) return false
		return !!(elem.offsetWidth || elem.offsetHeight || elem.getClientRects().length)
	},

	noop: function() {},

	/*
	 *  Filter the list of object such that only items that match all the given property values are returned.
	 */
	where: function(list, properties) {

		return list.filter(function(l) {
			if(Object.keys(properties).find(function(k){
				return l[k] === properties[k]
			})) return l;
		})
	},

	/*
	 *  As Utils.where above but only returns the first item
	 */
	findWhere: function(list, properties) {

		return Utils.where(list, properties)[0];
	},

	css_name: function(name){
		return name.replace(' ', '-').toLowerCase();
	},

	css: function(el, obj) {
		for(var k in obj){
			el.style[k] = obj[k]
		}
	},

	/*
	 *   From jquery
	 *
	 *   offset() relates an element's border box to the document origin
	 */
	offset: function(elem) {
		if(arguments.length > 1) throw new Error('Utils.offset: not supported')

		// Return zeros for disconnected and hidden (display: none) elements (gh-2310)
		// Support: IE <=11 only
		// Running getBoundingClientRect on a
		// disconnected node in IE throws an error
		if (!elem.getClientRects().length) {
			return {top: 0, left: 0};
		}

		// Get document-relative position by adding viewport scroll to viewport-relative gBCR
		var rect = elem.getBoundingClientRect();
		var win = elem.ownerDocument.defaultView;
		return {
			top: rect.top + win.pageYOffset,
			left: rect.left + win.pageXOffset
		}
	},

	/*
	 *   From jquery
	 *
	 *   position() relates an element's margin box to its offset parent's padding box
	 *   This corresponds to the behavior of CSS absolute positioning
	 */
	position: function(elem) {
		var offsetParent, offset, doc,
			parentOffset = { top: 0, left: 0 };

		// position:fixed elements are offset from the viewport, which itself always has zero offset
		if (elem.style.position === 'fixed') {

			// Assume position:fixed implies availability of getBoundingClientRect
			offset = elem.getBoundingClientRect();

		} else {
			offset = Utils.offset(elem);

			// Account for the *real* offset parent, which can be the document or its root element
			// when a statically positioned element is identified
			doc = elem.ownerDocument;
			offsetParent = elem.offsetParent || doc.documentElement;
			while (offsetParent && (offsetParent === doc.body || offsetParent === doc.documentElement) && offsetParent.style.position === 'static') {

				offsetParent = offsetParent.parentNode;
			}
			if (offsetParent && offsetParent !== elem && offsetParent.nodeType === 1) {

				// Incorporate borders into its offset, since they are outside its content origin
				parentOffset = Utils.offset(offsetParent)
				parentOffset.top += offsetParent.style.borderTopWidth
				parentOffset.left += offsetParent.style.borderLeftWidth
			}
		}

		// Subtract parent offsets and element margins
		return {
			top: offset.top - parentOffset.top - elem.style.marginTop,
			left: offset.left - parentOffset.left - elem.style.marginLeft
		}
	},

	// This method will return documentElement in the following cases:
	// 1) For the element inside the iframe without offsetParent, this method will return
	//    documentElement of the parent window
	// 2) For the hidden or detached element
	// 3) For body or html element, i.e. in case of the html node - it will return itself
	//
	// but those exceptions were never presented as a real life use-cases
	// and might be considered as more preferable results.
	//
	// This logic, however, is not guaranteed and can change at any point in the future
	offsetParent: function() {
		return this.map( function() {
			var offsetParent = this.offsetParent;

			while (offsetParent && offsetParent.style.position === 'static') {
				offsetParent = offsetParent.offsetParent;
			}

			return offsetParent || document.documentElement;
		})
	},

	once: function(el, event, callback) {
		function inner() {
			callback.apply(this, arguments)
			el.removeEventListener(event, inner)
		}
		el.addEventListener(event, inner)
	},

	/*
	 *  Delay execution of a fn and ensure it is only called once within a time window.
	 *  The fn will never execute while activity continues
	 *  It is different to throttle-debounce which executes the fn as soon as possible.
	 */
	debounce: function(t, fn) {
		t = t || 1000;
		var throttle_timer

		return function() {
			var self = this;

			throttle_timer && clearTimeout(throttle_timer)
			throttle_timer = setTimeout(function(){
				throttle_timer = null
				fn.apply(self)
			}, t)
		}
	}
}
extend(true, ECMWF, Utils);


/*
 *  Return an array of query parameters for the current page.
 */
ECMWF.query = function(){
    var query_string = {};
    var query = window.location.search.substring(1);
    var vars = query.split('&');
    for (var i=0;i<vars.length;i++) {
        var pair = vars[i].split('=');
        // If first entry with this name
        if (typeof query_string[pair[0]] === 'undefined') {
            pair[0] && (query_string[pair[0]] = pair[1]);
            // If second entry with this name
        } else if (typeof query_string[pair[0]] === 'string') {
            var arr = [ query_string[pair[0]], pair[1] ];
            query_string[pair[0]] = arr;
            // If third or later entry with this name
        } else {
            query_string[pair[0]].push(pair[1]);
        }
    }
    return query_string;
}

ECMWF.get_host = function() {
	return ECMWF.fix_host.to_backend(location.protocol + '//' + location.hostname + (location.port ? ':' + location.port : ''));
}

ECMWF.fix_host = {
	to_backend: function(url) {
		return url.replace(/www|eccharts/gi, 'apps').replace(/stage|eccharts-test/gi, 'apps-test');
	},
	to_frontend: function(url) {
		return url.replace(/apps-test|eccharts-test/gi, 'stage').replace(/apps|eccharts/gi, 'www');
	}
};

return Utils;
});

