import extend from 'extend'
import h from 'virtual-dom/h'
import Unit from 'unit'
import Utils from 'utils'
import Handler from './handler'
import CityAutocomplete from 'city-autocomplete/2'
import {Preferences} from 'settings'

function Location(options) {
	var self = this

	var o = this.options = extend({
		value: null
	}, options)

	this._value = extend({}, o.value || Preferences.get('default_location'));

	this.autocomplete = new CityAutocomplete({
		id: 'cities',
		value: this._value.station_name || this._value.title,
		select: function(pt) {
			self._value = extend({}, self._value, {
				// TODO just extend with pt?
				lat: pt.lat,
				lon: pt.lon,
				alt: pt.altitude,
				station_name: pt.title
			})
			o.renderer.render()
		},
		renderer: o.renderer
	})
}

Location.prototype = Utils.extend(new Handler(), {
	render: function() {
		var self = this

		return h('.form-location', [
			h('label', 'Latitude'),
			h('input.form-text.lat', {type: 'text', value: Unit.formatLatitude(this._value.lat, true), onchange: function(e){
				self._value.lat = e.target.value
			}}),
			h('label', 'Longitude'),
			h('input.form-text.lon', {type: 'text', value: Unit.formatLongitude(this._value.lon, true), onchange: function(e){
				self._value.lon = e.target.value
			}}),
			h('label', 'Altitude'),
			h('input.form-text', {type: 'text', disabled: true}),
			h('label', 'City'),
			h('#cities', this.autocomplete.render()),
			h('p.location-help', 'Please note that when using the city finder the lat/lon will not be updated unless an item is selected from the drop-down list, otherwise you can use the text purely as a label.')
		])
	},

	collect: function() {
		return this.options.value = {
			lat: Unit.parseLL(this._value.lat),
			lon: Unit.parseLL(this._value.lon),
			// TODO use either city or station_name, not both
			city: this.autocomplete.value,
			station_name: this.autocomplete.value,
			altitude: this._value.alt
		}
	},

	cancel: function() {
		this._value = this.options.value;
		this.autocomplete.options.value = this._value.station_name
	}
})

export default Location
